@@ -1,19 +1,17 @@
-
 import copy
 import json
 import logging
-from collections import MutableMapping
+from collections.abc import MutableMapping
 from urllib.parse import parse_qs
 from urllib.parse import urlencode
 
-from cryptojwt import as_unicode
-from cryptojwt import jwe
-from cryptojwt import jws
-from cryptojwt.jwk import keyitems2keyreps
-from cryptojwt.jws import NoSuitableSigningKeys
-from cryptojwt.jwe import JWE
-from cryptojwt.jws import JWS
-from cryptojwt import SimpleJWT
+from cryptojwt.jwe.jwe import JWE
+from cryptojwt.jwe.jwe import factory as jwe_factory
+from cryptojwt.jws.exception import NoSuitableSigningKeys
+from cryptojwt.jws.jws import JWS
+from cryptojwt.jws.jws import factory as jws_factory
+from cryptojwt.key_jar import update_keyjar
+from cryptojwt.utils import as_unicode
 
 from oidcmsg.exception import DecodeError
 from oidcmsg.exception import FormatError
@@ -22,11 +20,7 @@ from oidcmsg.exception import MissingRequiredAttribute
 from oidcmsg.exception import MissingSigningKey
 from oidcmsg.exception import NotAllowedValue
 from oidcmsg.exception import OidcMsgError
-from oidcmsg.exception import ParameterError
 from oidcmsg.exception import TooManyValues
-from oidcmsg.exception import WrongEncryptionAlgorithm
-from oidcmsg.exception import WrongSigningAlgorithm
-from oidcmsg.key_jar import update_keyjar
 
 logger = logging.getLogger(__name__)
 
@@ -35,14 +29,17 @@ ERRTXT = "On '%s': %s"
 
 class Message(MutableMapping):
     """
-    Represents a basic protocol nessage/item in OAuth2/OIDC 
+    Represents a basic protocol nessage/item in OAuth2/OIDC
     """
     c_param = {}
     c_default = {}
     c_allowed_values = {}
 
-    def __init__(self, **kwargs):
-        self._dict = self.c_default.copy()
+    def __init__(self, set_defaults=True, **kwargs):
+        if set_defaults:
+            self._dict = self.c_default.copy()
+        else:
+            self._dict = {}
         self.lax = False
         self.jwt = None
         self.jws_header = None
@@ -53,7 +50,7 @@ class Message(MutableMapping):
     def __iter__(self):
         """
         Returns an iterator over all the key, value pairs in this class instance
-        
+
         :return: iterator
         """
         return iter(self._dict)
@@ -61,16 +58,16 @@ class Message(MutableMapping):
     def type(self):
         """
         Return the type of protocol message this is
-        
-        :return: The name of the message 
+
+        :return: The name of the message
         """
         return self.__class__.__name__
 
     def parameters(self):
         """
         Returns a list of all known parameters for this message type.
-        
-        :return: list of parameter names 
+
+        :return: list of parameter names
         """
         return list(self.c_param.keys())
 
@@ -79,7 +76,7 @@ class Message(MutableMapping):
         Based on specification set a parameters value to the default value.
         """
         for key, val in self.c_default.items():
-            self._dict[key] = val
+            self._dict.setdefault(key, val)
 
     def to_urlencoded(self, lev=0):
         """
@@ -90,7 +87,7 @@ class Message(MutableMapping):
 
         _spec = self.c_param
         if not self.lax:
-            for attribute, (_, req, _ser, _, na) in _spec.items():
+            for attribute, (_, req, _ser, _, _) in _spec.items():
                 if req and attribute not in self._dict:
                     raise MissingRequiredAttribute("%s" % attribute,
                                                    "%s" % self)
@@ -102,7 +99,7 @@ class Message(MutableMapping):
                 (_, req, _ser, _, null_allowed) = _spec[key]
             except KeyError:  # extra attribute
                 try:
-                    _key, lang = key.split("#")
+                    _key = key.split("#")[0]
                     (_, req, _ser, _deser, null_allowed) = _spec[_key]
                 except (ValueError, KeyError):
                     try:
@@ -113,7 +110,8 @@ class Message(MutableMapping):
 
             if val is None and null_allowed is False:
                 continue
-            elif isinstance(val, str):
+
+            if isinstance(val, str):
                 # Should I allow parameters with "" as value ???
                 params.append((key, val.encode("utf-8")))
             elif isinstance(val, list):
@@ -150,12 +148,12 @@ class Message(MutableMapping):
 
     def serialize(self, method="urlencoded", lev=0, **kwargs):
         """
-        Convert this instance to another representation. Which representation 
+        Convert this instance to another representation. Which representation
         is given by the choice of serialization method.
-        
+
         :param method: A serialization method. Presently 'urlencoded', 'json',
             'jwt' and 'dict' is supported.
-        :param lev: 
+        :param lev:
         :param kwargs: Extra key word arguments
         :return: THe content of this message serialized using a chosen method
         """
@@ -164,8 +162,8 @@ class Message(MutableMapping):
     def deserialize(self, info, method="urlencoded", **kwargs):
         """
         Convert from an external representation to an internal.
-        
-        :param info: The input  
+
+        :param info: The input
         :param method: The method used to deserialize the info
         :param kwargs: extra Keyword arguments
         :return: In the normal case the Message instance
@@ -179,7 +177,7 @@ class Message(MutableMapping):
 
     def from_urlencoded(self, urlencoded, **kwargs):
         """
-        Starting with a string of the application/x-www-form-urlencoded format 
+        Starting with a string of the application/x-www-form-urlencoded format
         this method creates a class instance
 
         :param urlencoded: The string
@@ -197,16 +195,20 @@ class Message(MutableMapping):
 
         _spec = self.c_param
 
-        for key, val in parse_qs(urlencoded).items():
+        _info = parse_qs(urlencoded)
+        if len(urlencoded) and _info == {}:
+            raise FormatError('Wrong format')
+
+        for key, val in _info.items():
             try:
-                (typ, _, _, _deser, null_allowed) = _spec[key]
+                (typ, _, _, _deser, _) = _spec[key]
             except KeyError:
                 try:
-                    _key, lang = key.split("#")
-                    (typ, _, _, _deser, null_allowed) = _spec[_key]
+                    _key = key.split("#")[0]
+                    (typ, _, _, _deser, _) = _spec[_key]
                 except (ValueError, KeyError):
                     try:
-                        (typ, _, _, _deser, null_allowed) = _spec['*']
+                        (typ, _, _, _deser, _) = _spec['*']
                     except KeyError:
                         if len(val) == 1:
                             val = val[0]
@@ -226,10 +228,7 @@ class Message(MutableMapping):
                     elif isinstance(val[0], typ):
                         self._dict[key] = val[0]
                     else:
-                        try:
-                            self._dict[key] = val[0]
-                        except KeyError:
-                            raise ParameterError(key)
+                        self._dict[key] = val[0]
                 else:
                     raise TooManyValues('{}'.format(key))
 
@@ -248,14 +247,14 @@ class Message(MutableMapping):
         lev += 1
         for key, val in self._dict.items():
             try:
-                (_, req, _ser, _, null_allowed) = _spec[str(key)]
+                _ser = _spec[str(key)][2]
             except KeyError:
                 try:
-                    _key, lang = key.split("#")
-                    (_, req, _ser, _, null_allowed) = _spec[_key]
+                    _key = key.split("#")[0]
+                    _ser = _spec[_key][2]
                 except (ValueError, KeyError):
                     try:
-                        (_, req, _ser, _, null_allowed) = _spec['*']
+                        _ser = _spec['*'][2]
                     except KeyError:
                         _ser = None
 
@@ -286,16 +285,16 @@ class Message(MutableMapping):
         for key, val in dictionary.items():
             # Earlier versions of python don't like unicode strings as
             # variable names
-            if val == "" or val == [""]:
+            if val in ["", [""]]:
                 continue
 
             skey = str(key)
             try:
-                (vtyp, req, _, _deser, null_allowed) = _spec[key]
+                (vtyp, _, _, _deser, null_allowed) = _spec[key]
             except KeyError:
                 # might be a parameter with a lang tag
                 try:
-                    _key, lang = skey.split("#")
+                    _key = skey.split("#")[0]
                 except ValueError:
                     try:
                         (vtyp, _, _, _deser, null_allowed) = _spec['*']
@@ -307,7 +306,7 @@ class Message(MutableMapping):
                         continue
                 else:
                     try:
-                        (vtyp, req, _, _deser, null_allowed) = _spec[_key]
+                        (vtyp, _, _, _deser, null_allowed) = _spec[_key]
                     except KeyError:
                         try:
                             (vtyp, _, _, _deser, null_allowed) = _spec['*']
@@ -325,8 +324,8 @@ class Message(MutableMapping):
         """
         Main method for adding a value to the instance. Does all the
         checking on type of value and if among allowed values.
-        
-        :param skey: string version of the key 
+
+        :param skey: string version of the key
         :param vtyp: Type of value
         :param key: original representation of the key
         :param val: The value to add
@@ -342,6 +341,12 @@ class Message(MutableMapping):
             vtyp = vtyp[0]
 
         if isinstance(vtyp, list):
+            if val is None:
+                if null_allowed:
+                    self._dict[key] = val
+                    return
+                raise ValueError("Null is not allowed")
+
             vtype = vtyp[0]
             if isinstance(val, vtype):
                 if issubclass(vtype, Message):
@@ -364,8 +369,7 @@ class Message(MutableMapping):
                     try:
                         _val = []
                         for v in val:
-                            _val.append(vtype(**dict([(str(x), y) for x, y
-                                                      in v.items()])))
+                            _val.append(vtype(**{str(x): y for x, y in v.items()}))
                         val = _val
                     except Exception as exc:
                         raise DecodeError(ERRTXT % (key, exc))
@@ -373,8 +377,7 @@ class Message(MutableMapping):
                     for v in val:
                         if not isinstance(v, vtype):
                             raise DecodeError(
-                                ERRTXT % (key, "type != %s (%s)" % (
-                                    vtype, type(v))))
+                                ERRTXT % (key, "type != %s (%s)" % (vtype, type(v))))
 
                 self._dict[skey] = val
             elif isinstance(val, dict):
@@ -403,6 +406,18 @@ class Message(MutableMapping):
                         val = _deser(val, sformat="dict")
                     except Exception as exc:
                         raise DecodeError(ERRTXT % (key, exc))
+                    else:
+                        # if isinstance(val, str):
+                        #     self._dict[skey] = val
+                        # elif isinstance(val, list):
+                        #     if len(val) == 1:
+                        #         self._dict[skey] = val[0]
+                        #     elif not len(val):
+                        #         pass
+                        #     else:
+                        #         raise TooManyValues(key)
+                        # else:
+                        self._dict[skey] = val
                 elif vtyp is int:
                     try:
                         self._dict[skey] = int(val)
@@ -410,41 +425,29 @@ class Message(MutableMapping):
                         raise ValueError(
                             '"{}", wrong type of value for "{}"'.format(val,
                                                                         skey))
-                    else:
-                        return
                 elif vtyp is bool:
                     raise ValueError(
                         '"{}", wrong type of value for "{}"'.format(val, skey))
                 elif vtyp != type(val):
                     if vtyp == Message:
-                        if type(val) == dict or isinstance(val, str):
+                        if isinstance(val, (dict, str)):
                             self._dict[skey] = val
-                            return
                         else:
                             raise ValueError(
                                 '"{}", wrong type of value for "{}"'.format(
                                     val, skey))
-                    raise ValueError(
-                        '"{}", wrong type of value for "{}"'.format(val, skey))
-                if isinstance(val, str):
-                    self._dict[skey] = val
-                elif isinstance(val, list):
-                    if len(val) == 1:
-                        self._dict[skey] = val[0]
-                    elif not len(val):
-                        pass
                     else:
-                        raise TooManyValues(key)
-                else:
-                    self._dict[skey] = val
+                        raise ValueError(
+                            '"{}", wrong type of value for "{}"'.format(val,
+                                                                        skey))
 
     def to_json(self, lev=0, indent=None):
         """
         Serialize the content of this instance into a JSON string.
-        
-        :param lev: 
-        :param indent: Number of spaces that should be used for indentation 
-        :return: 
+
+        :param lev:
+        :param indent: Number of spaces that should be used for indentation
+        :return:
         """
         if lev:
             return self.to_dict(lev + 1)
@@ -454,16 +457,13 @@ class Message(MutableMapping):
     def from_json(self, txt, **kwargs):
         """
         Convert from a JSON string to an instance of this class.
-        
-        :param txt: The JSON string 
-        :param kwargs: extra keyword arguments
-        :return: The instantiated instance 
-        """
-        try:
-            _dict = json.loads(txt)
-        except TypeError:
-            _dict = json.loads(as_unicode(txt))
 
+        :param txt: The JSON string (a ``str``, ``bytes`` or ``bytearray``
+            instance containing a JSON document)
+        :param kwargs: extra keyword arguments
+        :return: The instantiated instance
+        """
+        _dict = json.loads(txt)
         return self.from_dict(_dict)
 
     def to_jwt(self, key=None, algorithm="", lev=0, lifetime=0):
@@ -494,76 +494,66 @@ class Message(MutableMapping):
         :return: A class instance
         """
 
-        _jw = jwe.factory(txt)
-        if _jw:
-            logger.debug("JWE headers: {}".format(_jw.jwt.headers))
+        algarg = {}
+        if 'encalg' in kwargs:
+            algarg['alg'] = kwargs['encalg']
+        if 'encenc' in kwargs:
+            algarg['enc'] = kwargs['encenc']
+        _decryptor = jwe_factory(txt, **algarg)
 
-            if "encalg" in kwargs:
-                if kwargs["encalg"] != _jw["alg"]:
-                    raise WrongEncryptionAlgorithm("%s != %s" % (
-                        _jw["alg"], ["encalg"]))
-
-                if kwargs["encenc"] != _jw["enc"]:
-                    raise WrongEncryptionAlgorithm("%s != %s" % (
-                        _jw["enc"], kwargs["encenc"]))
+        if _decryptor:
+            logger.debug("JWE headers: %s", _decryptor.jwt.headers)
 
             dkeys = keyjar.get_decrypt_key(owner="")
-            # if "sender" in kwargs:
-            #     dkeys.extend(keyjar.get_deccrypt_key(owner=kwargs["sender"]))
 
-            logger.debug('Decrypt class: {}'.format(_jw.__class__))
-            _res = _jw.decrypt(txt, dkeys)
-            logger.debug('decrypted message:{}'.format(_res))
+            logger.debug('Decrypt class: %s', _decryptor.__class__)
+            _res = _decryptor.decrypt(txt, dkeys)
+            logger.debug('decrypted message: %s', _res)
             if isinstance(_res, tuple):
                 txt = as_unicode(_res[0])
             elif isinstance(_res, list) and len(_res) == 2:
                 txt = as_unicode(_res[0])
             else:
                 txt = as_unicode(_res)
-            self.jwe_header = _jw.jwt.headers
+            self.jwe_header = _decryptor.jwt.headers
 
-        _jw = jws.factory(txt)
-        if _jw:
-            if "sigalg" in kwargs:
-                _alg = _jw.jwt.headers["alg"]
-                if kwargs["sigalg"] != _alg:
-                    raise WrongSigningAlgorithm("%s != %s" % (
-                        _alg, kwargs["sigalg"]))
-            try:
-                _jwt = SimpleJWT().unpack(txt)
-                jso = _jwt.payload()
-                _header = _jwt.headers
+        if kwargs.get('sigalg'):
+            _verifier = jws_factory(txt, alg=kwargs['sigalg'])
+        else:
+            _verifier = jws_factory(txt)
 
-                key = []
+        if _verifier:
+            _jwt = _verifier.jwt
+            jso = _jwt.payload()
+            _header = _jwt.headers
 
-                # if "sender" in kwargs:
-                #     key.extend(keyjar.get_verify_key(owner=kwargs["sender"]))
+            key = []
+            # if "sender" in kwargs:
+            #     key.extend(keyjar.get_verify_key(owner=kwargs["sender"]))
 
-                logger.debug("Raw JSON: {}".format(jso))
-                logger.debug("JWS header: {}".format(_header))
-                if _header["alg"] == "none":
-                    pass
-                elif verify:
+            logger.debug("Raw JSON: %s", jso)
+            logger.debug("JWS header: %s", _header)
+            if _header["alg"] == "none":
+                pass
+            elif verify:
+                if keyjar:
+                    key.extend(keyjar.get_jwt_verify_keys(_jwt, **kwargs))
+
+                if "alg" in _header and _header["alg"] != "none":
+                    if not key:
+                        raise MissingSigningKey(
+                            "alg=%s" % _header["alg"])
+
+                logger.debug("Found signing key.")
+                try:
+                    _verifier.verify_compact(txt, key)
+                except NoSuitableSigningKeys:
                     if keyjar:
-                        key.extend(keyjar.get_jwt_verify_keys(_jwt, **kwargs))
+                        update_keyjar(keyjar)
+                        key = keyjar.get_jwt_verify_keys(_jwt, **kwargs)
+                        _verifier.verify_compact(txt, key)
 
-                    if "alg" in _header and _header["alg"] != "none":
-                        if not key:
-                            raise MissingSigningKey(
-                                "alg=%s" % _header["alg"])
-
-                    logger.debug("Found signing key.")
-                    try:
-                        _jw.verify_compact(txt, key)
-                    except NoSuitableSigningKeys:
-                        if keyjar:
-                            update_keyjar(keyjar)
-                            key = keyjar.get_jwt_verify_keys(_jwt, **kwargs)
-                            _jw.verify_compact(txt, key)
-            except Exception:
-                raise
-            else:
-                self.jws_header = _jwt.headers
+            self.jws_header = _jwt.headers
         else:
             jso = json.loads(txt)
 
@@ -573,26 +563,28 @@ class Message(MutableMapping):
     def __str__(self):
         """
         Return a string representation of this class
-        
-        :return: A string representation of this class 
+
+        :return: A string representation of this class
         """
         return '{}'.format(self.to_dict())
 
-    def _type_check(self, typ, _allowed, val, na=False):
+    @staticmethod
+    def _type_check(typ, _allowed, val, na=False):
         if typ is str:
             if val not in _allowed:
-                raise NotAllowedValue(val)
+                return False
         elif typ is int:
             if val not in _allowed:
-                raise NotAllowedValue(val)
+                return False
         elif isinstance(typ, list):
             if isinstance(val, list):
                 # _typ = typ[0]
                 for item in val:
                     if item not in _allowed:
-                        raise NotAllowedValue(val)
+                        return False
         elif val is None and na is False:
-            raise NotAllowedValue(val)
+            return False
+        return True
 
     def verify(self, **kwargs):
         """
@@ -628,7 +620,8 @@ class Message(MutableMapping):
             except KeyError:
                 pass
             else:
-                self._type_check(typ, _allowed_val, val, na)
+                if not self._type_check(typ, _allowed_val, val, na):
+                    raise NotAllowedValue(val)
 
         return True
 
@@ -644,9 +637,9 @@ class Message(MutableMapping):
     def __getitem__(self, item):
         """
         Return the value of a specified parameter.
-        
-        :param item: 
-        :return: 
+
+        :param item:
+        :return:
         """
         return self._dict[item]
 
@@ -654,11 +647,11 @@ class Message(MutableMapping):
         """
         Return the value of a specific parameter. If the parameter does not
         have a value return the default value.
-        
-        :param item: The name of the parameter 
+
+        :param item: The name of the parameter
         :param default: Default value
-        :return: The value of the parameter or, if that doesn't exist, 
-            the default value 
+        :return: The value of the parameter or, if that doesn't exist,
+            the default value
         """
         try:
             return self[item]
@@ -669,8 +662,8 @@ class Message(MutableMapping):
         """
         Return a list of tuples (key, value) representing all parameters
         of this class instance that has a value.
-        
-        :return: iterator 
+
+        :return: iterator
         """
         return self._dict.items()
 
@@ -680,8 +673,8 @@ class Message(MutableMapping):
     def __contains__(self, item):
         """
         Answers the question: does this parameter have a value?
-        
-        :param item: The name of the parameter 
+
+        :param item: The name of the parameter
         :return: True/False
         """
         return item in self._dict
@@ -690,11 +683,11 @@ class Message(MutableMapping):
         """
         Given a URL this method will add a fragment, a query part or extend
         a query part if it already exists with the information in this instance.
-        
-        :param location: A URL 
+
+        :param location: A URL
         :param fragment_enc: Whether the information should be placed in a
             fragment (True) or in a query part (False)
-        :return: The extended URL 
+        :return: The extended URL
         """
         _l = as_unicode(location)
         _qp = as_unicode(self.to_urlencoded())
@@ -716,7 +709,7 @@ class Message(MutableMapping):
     def __eq__(self, other):
         """
         Compare two message instances. This with another instance.
-        
+
         :param other:  The other instance
         :return: True/False
         """
@@ -739,8 +732,8 @@ class Message(MutableMapping):
     def __len__(self):
         """
         Return the number of parameters that has a value.
-        
-        :return: Number of parameters with a value. 
+
+        :return: Number of parameters with a value.
         """
         return len(self._dict)
 
@@ -748,7 +741,7 @@ class Message(MutableMapping):
         """
         Return the extra parameters that this instance. Extra meaning those
         that are not listed in the c_params specification.
-        
+
         :return: The key,value pairs for keys that are not in the c_params
             specification,
         """
@@ -759,8 +752,8 @@ class Message(MutableMapping):
         """
         Return True if this instance only has key,value pairs for keys
         that are not defined in c_params.
-        
-        :return: True/False 
+
+        :return: True/False
         """
         known = [key for key in self._dict.keys() if key in self.c_param]
         if not known:
@@ -768,11 +761,11 @@ class Message(MutableMapping):
         else:
             return False
 
-    def update(self, item):
+    def update(self, item, **kwargs):
         """
         Update the information in this instance.
-        
-        :param item: a dictionary or a Message instance 
+
+        :param item: a dictionary or a Message instance
         """
         if isinstance(item, dict):
             self._dict.update(item)
@@ -788,16 +781,13 @@ class Message(MutableMapping):
         JSON object the body of a JWT. Then encrypt that JWT using the
         specified algorithms and the given keys. Return the encrypted JWT.
 
-        :param keys: Dictionary, keys are key type and key is the value or
-            simple list.
+        :param keys: list or KeyJar instance
         :param enc: Content Encryption Algorithm
         :param alg: Key Management Algorithm
         :param lev: Used for JSON construction
         :return: An encrypted JWT. If encryption failed an exception will be
             raised.
         """
-        if isinstance(keys, dict):
-            keys = keyitems2keyreps(keys)
 
         _jwe = JWE(self.to_json(lev), alg=alg, enc=enc)
         return _jwe.encrypt(keys)
@@ -808,12 +798,11 @@ class Message(MutableMapping):
         of the JWT into this object.
 
         :param msg: An encrypted JWT
-        :param keys: Dictionary, keys are key type and key is the value orsimple list.
+        :param keys: Possibly usable keys.
+        :type keys: list or KeyJar instance
         :return: The decrypted message. If decryption failed an exception
             will be raised.
         """
-        if isinstance(keys, dict):
-            keys = keyitems2keyreps(keys)
 
         jwe = JWE()
         _res = jwe.decrypt(msg, keys)
@@ -838,6 +827,24 @@ class Message(MutableMapping):
         for key in _blanks:
             del self._dict[key]
 
+    def required_parameters(self):
+        """
+        Return the required parameters for this type of message.
+        """
+        return [p for p, s in self.c_param.items() if s[1] is True]
+
+    def value_type(self, parameter):
+        """
+        Return the type of value that a parameter can have.
+
+        :param parameter: Name of the parameter
+        :return: Type of Value or None if unknown.
+        """
+        _spec = self.c_param.get(parameter)
+        if _spec:
+            return _spec[0]
+
+        return None
 
 # =============================================================================
 
@@ -960,6 +967,8 @@ SINGLE_REQUIRED_STRING = (str, True, None, None, False)
 SINGLE_OPTIONAL_STRING = (str, False, None, None, False)
 SINGLE_OPTIONAL_INT = (int, False, None, None, False)
 SINGLE_REQUIRED_INT = (int, True, None, None, False)
+SINGLE_REQUIRED_BOOLEAN = (bool, True, None, None, False)
+
 OPTIONAL_LIST_OF_STRINGS = ([str], False, list_serializer,
                             list_deserializer, False)
 REQUIRED_LIST_OF_STRINGS = ([str], True, list_serializer,
@@ -971,6 +980,9 @@ REQUIRED_LIST_OF_SP_SEP_STRINGS = ([str], True, sp_sep_list_serializer,
 SINGLE_OPTIONAL_JSON = (dict, False, json_serializer, json_deserializer,
                         False)
 
+SINGLE_REQUIRED_JSON = (dict, True, json_serializer, json_deserializer,
+                        False)
+
 REQUIRED = [SINGLE_REQUIRED_STRING, REQUIRED_LIST_OF_STRINGS,
             REQUIRED_LIST_OF_SP_SEP_STRINGS]
 
@@ -979,3 +991,30 @@ REQUIRED_MESSAGE = (Message, True, msg_ser, msg_deser, False)
 
 OPTIONAL_LIST_OF_MESSAGES = ([Message], False, msg_list_ser, msg_list_deser,
                              False)
+
+
+def any_ser(val, sformat="urlencoded", lev=0):
+    if isinstance(val, (str, int, bool)):
+        return val
+    elif isinstance(val, Message):
+        return msg_ser(val, sformat, lev)
+    elif isinstance(val, dict):
+        return json.dumps(val)
+    elif isinstance(val, list):
+        return msg_list_ser(val)
+    else:
+        raise ValueError("Can't serialize this type of data")
+
+
+def any_deser(val, sformat="urlencoded", lev=0):
+    if isinstance(val, dict):
+        return Message(**val)
+    elif isinstance(val, list):
+        return [msg_deser(v, sformat) for v in val]
+    else:
+        raise ValueError("Can't deserialize this type of data")
+
+
+SINGLE_OPTIONAL_ANY = (any, False, any_ser, any_deser, False)
+REQUIRED_LIST_OF_DICTS = ([dict], True, list_serializer, list_deserializer, False)
+OPTIONAL_LIST_OF_DICTS = ([dict], False, list_serializer, list_deserializer, False)
